@@ -30,6 +30,7 @@
 (use-package '("BDB") "EXT")
 (ext:re-export "BDB" "EXT")
 (pushnew :berkeley-db *features*)
+(provide "berkeley-db")
 (in-package "BDB")
 
 (setf (documentation (find-package "BDB") 'sys::impnotes) "berkeley-db")
@@ -347,7 +348,7 @@
 )
 
 (define-condition bdb-error (simple-error)
-  (($errno :initarg :errno :reader bdb-error-number)))
+  (($ecode :reader bdb-error-code :initarg :code)))
 
 ;;; restore locks
 (pushnew "BDB" custom:*system-package-list* :test #'string=)
