@@ -101,7 +101,7 @@ DEFMODULE(rawsock,"RAWSOCK")
 static object my_check_argument (object name, object datum) {
   pushSTACK(NIL);               /* no PLACE */
   pushSTACK(name); pushSTACK(datum); pushSTACK(TheSubr(subr_self)->name);
-  check_value(error,GETTEXT("~S: ~S is not a valid ~S argument"));
+  check_value(error_condition,GETTEXT("~S: ~S is not a valid ~S argument"));
   return value1;
 }
 /* DANGER: the return value is invalidated by GC!
@@ -405,13 +405,13 @@ DEFUN(RAWSOCK:CONVERT-ADDRESS, family address) {
       } break;
       default: value1 = NIL;
     }
-  } else fehler_string_integer(STACK_0);
+  } else error_string_integer(STACK_0);
   if (nullp(value1)) {
     pushSTACK(NIL);             /* no PLACE */
     pushSTACK(STACK_1);         /* domain */
     pushSTACK(STACK_1);         /* address */
     pushSTACK(TheSubr(subr_self)->name);
-    check_value(error,GETTEXT("~S: invalid address ~S for family ~S"));
+    check_value(error_condition,GETTEXT("~S: invalid address ~S for family ~S"));
     STACK_0 = value1;
     goto convert_address_restart;
   }
@@ -461,7 +461,7 @@ DEFUN(RAWSOCK:PROTOCOL, &optional protocol)
         end_system_call();
       });
 #  endif
-  } else fehler_string_integer(proto);
+  } else error_string_integer(proto);
   if (pe) protoent_to_protocol(pe);
   else VALUES1(NIL);
 }
@@ -511,7 +511,7 @@ DEFUN(RAWSOCK:NETWORK, &optional network type)
         end_system_call();
       });
 #  endif
-  } else fehler_string_integer(net);
+  } else error_string_integer(net);
   if (ne) netent_to_network(ne);
   else VALUES1(NIL);
 }
@@ -544,7 +544,7 @@ DEFUN(RAWSOCK:IF-NAME-INDEX, &optional what) {
     VALUES1(asciz_to_string(name,GLO(misc_encoding)));
 #  else
     pushSTACK(TheSubr(subr_self)->name);
-    fehler(error,GETTEXT("~S: no if_indextoname() at configure time"));
+    error(error_condition,GETTEXT("~S: no if_indextoname() at configure time"));
 #  endif
   } else if (stringp(STACK_0)) {
 #  if defined(HAVE_IF_INDEXTONAME)
@@ -557,9 +557,9 @@ DEFUN(RAWSOCK:IF-NAME-INDEX, &optional what) {
     VALUES1(uint_to_I(idx));
 #  else
     pushSTACK(TheSubr(subr_self)->name);
-    fehler(error,GETTEXT("~S: no if_nametoindex() at configure time"));
+    error(error_condition,GETTEXT("~S: no if_nametoindex() at configure time"));
 #  endif
-  } else fehler_string_integer(STACK_0);
+  } else error_string_integer(STACK_0);
   skipSTACK(1);
 }
 #endif  /* net/if.h */
@@ -582,7 +582,7 @@ DEFUN(RAWSOCK:IFADDRS,) {
     if (ifap->ifa_flags & IFF_BROADCAST)
       if (ifap->ifa_flags & IFF_POINTOPOINT) {
         pushSTACK(TheSubr(subr_self)->name);
-        fehler(error,GETTEXT("~S: both IFF_BROADCAST and IFF_POINTOPOINT set"));
+        error(error_condition,GETTEXT("~S: both IFF_BROADCAST and IFF_POINTOPOINT set"));
       } else pushSTACK(ifap->ifa_broadaddr
                        ? sockaddr_to_lisp1(ifap->ifa_broadaddr) : NIL);
     else if (ifap->ifa_flags & IFF_POINTOPOINT)
@@ -636,7 +636,7 @@ static int get_socket_protocol (object proto) {
     if (pe) return pe->p_proto;
     pushSTACK(NIL);             /* no PLACE */
     pushSTACK(TheSubr(subr_self)->name); pushSTACK(proto);
-    check_value(error,GETTEXT("~S: invalid protocol name ~S"));
+    check_value(error_condition,GETTEXT("~S: invalid protocol name ~S"));
     proto = value1;
     goto get_socket_protocol_restart;
   } else
@@ -772,7 +772,7 @@ DEFUN(RAWSOCK:SOCK-LISTEN,socket &optional backlog) {
 #if defined(WIN32_NATIVE)
 nonreturning_function(static, error_missing, (object function)) {
   pushSTACK(function); pushSTACK(TheSubr(subr_self)->name);
-  fehler(error,GETTEXT("~S: your ws2_32.dll does not implement ~S"));
+  error(error_condition,GETTEXT("~S: your ws2_32.dll does not implement ~S"));
 }
 #endif
 
@@ -1365,7 +1365,7 @@ static void set_sock_opt_many (rawsock_t sock, int level, int name,
     while (!endp(STACK_0)) {
       int name = sockopt_name(Car(STACK_0));
       STACK_0 = Cdr(STACK_0);
-      if (!consp(STACK_0)) fehler_plist_odd(STACK_1);
+      if (!consp(STACK_0)) error_plist_odd(STACK_1);
       set_sock_opt(sock,level,name,Car(STACK_0));
       STACK_0 = Cdr(STACK_0);
     }
@@ -1385,7 +1385,7 @@ DEFUN(RAWSOCK::SET-SOCKET-OPTION, value sock name &key :LEVEL)
     while (!endp(STACK_0)) {
       int level = sockopt_level(Car(STACK_0));
       STACK_0 = Cdr(STACK_0);
-      if (!consp(STACK_0)) fehler_plist_odd(STACK_1);
+      if (!consp(STACK_0)) error_plist_odd(STACK_1);
       set_sock_opt_many(sock,level,name,Car(STACK_0));
       STACK_0 = Cdr(STACK_0);
     }
