@@ -109,6 +109,19 @@ to print the corresponding values, or T for all of them.")
             (ext::browse-url doc :out stream)
             (format stream name doc))))))
 
+(defvar *squeeze-string-max* 100)
+(defvar *squeeze-string-section* 10) ; at most half of *squeeze-string-max*
+(defun squeeze-string (string &key (start 0) (end (length string)))
+  (let* ((len (length string))
+         (prefix (if (= start 0) "" "..."))
+         (suffix (if (= end len) "" "...")))
+    (cond ((> *squeeze-string-max* len) string)
+          ((> *squeeze-string-max* (- end start))
+           (string-concat prefix (subseq string start end) suffix))
+          (t (string-concat
+              prefix (subseq string start (+ start *squeeze-string-section*))
+              "..." (subseq string (- end *squeeze-string-section*)) suffix)))))
+
 (clos:defgeneric describe-object (obj stream)
   (:method ((obj t) (stream stream))
     (ecase (type-of obj)
@@ -138,11 +151,11 @@ to print the corresponding values, or T for all of them.")
        (format stream (TEXT "a global symbol macro handler.")))
       (SYS::MACRO
        (format stream (TEXT "a macro expander."))
-       (terpri stream)
-       (format stream (TEXT "For more information, evaluate ~{~S~^ or ~}.")
-               `((DISASSEMBLE (MACRO-FUNCTION
-                               ',(sys::closure-name
-                                  (sys::macro-expander obj)))))))
+       (let ((name (sys::closure-name (sys::macro-expander obj))))
+         (describe-arglist stream name)
+         (terpri stream)
+         (format stream (TEXT "For more information, evaluate ~{~S~^ or ~}.")
+                 `((DISASSEMBLE (MACRO-FUNCTION ',name))))))
       (EXT:FUNCTION-MACRO
        (format stream (TEXT "a function with alternative macro expander.")))
       (EXT:ENCODING
@@ -377,7 +390,15 @@ to print the corresponding values, or T for all of them.")
   (:method ((obj stream) (stream stream))
     (format stream (TEXT "a~:[~:[ closed ~;n output-~]~;~:[n input-~;n input/output-~]~]stream.")
             (and (input-stream-p obj) (open-stream-p obj))
-            (and (output-stream-p obj) (open-stream-p obj))))
+            (and (output-stream-p obj) (open-stream-p obj)))
+    (when (sys::string-stream-p obj)
+      (multiple-value-bind (string start end) (sys::string-stream-string obj)
+        (if (and start end)     ; STRING-INPUT-STREAM
+            (format stream (TEXT " It reads from ~S from ~:D to ~:D at ~:D.")
+                    (squeeze-string string :start start :end end)
+                    start end (sys::string-input-stream-index obj))
+            (format stream (TEXT " It appends to ~S.")
+                    (squeeze-string string))))))
   (:method ((obj package) (stream stream))
     (if (package-name obj)
       (progn
@@ -549,10 +570,12 @@ to print the corresponding values, or T for all of them.")
     (fresh-line stream)
     (terpri stream) ; blank line
     (if (memq obj *describe-done*)
-      (format stream (TEXT "~A [see above]") objstring)
+      (with-stream-s-expression (stream)
+        (format stream (TEXT "~A [see above]") objstring))
       (let ((doc (and (symbolp obj) (get obj 'sys::doc))))
         (push obj *describe-done*)
-        (format stream (TEXT "~A is ") objstring)
+        (with-stream-s-expression (stream)
+          (format stream (TEXT "~A is ") objstring))
         (describe-object obj stream)
         (fresh-line stream)
         (when doc
@@ -594,8 +617,10 @@ to print the corresponding values, or T for all of them.")
 ;; auxiliary functions for DESCRIBE of FUNCTION
 
 (defun arglist (func)
-  (setq func (coerce func 'function))
-  (cond ((typep func 'generic-function)
+  (cond ((and (symbolp func) (fboundp func)
+              (sys::macrop (symbol-function func)))
+         (sys::macro-lambda-list (symbol-function func)))
+        ((typep (setq func (coerce func 'function)) 'generic-function)
          ;; Generic functions store the original lambda-list.
          (clos:generic-function-lambda-list func))
         ((or (sys::subr-info func) ; built-in
@@ -609,7 +634,9 @@ to print the corresponding values, or T for all of them.")
 
 (defun describe-arglist (stream function)
   (terpri stream)
-  (format stream (TEXT "Argument list: ~:S") (arglist function)))
+  (format stream (TEXT "Argument list: ~:S")
+          (handler-case (arglist function)
+            (error (c) (princ-to-string c)))))
 
 (defun describe-documentation (stream function)
   (let ((doc (clos::function-documentation function)))
