@@ -1,10 +1,10 @@
 (in-package "EXT")
-(export '(ethe letf letf* with-collect))
+(export '(ethe letf letf* with-collect compiled-file-p))
 (in-package "SYSTEM")
 ;;; ---------------------------------------------------------------------------
 ;;; Wie THE, nur dass auch im compilierten Code der Typtest durchgeführt wird.
 (defmacro ethe (typespec form)
-  (let ((g (gensym)))
+  (let ((g (gensym "VALUES-")))
     `(THE ,typespec
        (LET ((,g (MULTIPLE-VALUE-LIST ,form)))
          (IF (SYS::%THE ,g ',(type-for-discrimination typespec))
@@ -47,25 +47,29 @@
 ; (LETF (((VALUES-LIST A) form)) ...)
 ;   --> (LET ((A (MULTIPLE-VALUE-LIST form))) ...)
 
-(defmacro LETF* (bindlist &body body)
+(defmacro LETF* (&whole whole-form
+                 bindlist &body body &environment env)
   (multiple-value-bind (body-rest declarations) (SYSTEM::PARSE-BODY body)
     (let ((declare (if declarations `((DECLARE ,@declarations)) '())))
-      (values (expand-LETF* bindlist declare body-rest)))))
+      (values (expand-LETF* bindlist declare body-rest whole-form env)))))
 
 ; expandiert ein LETF*, liefert die Expansion und
 ; T, falls diese Expansion mit einem LET* anfängt, dessen Bindungsliste
 ; erweitert werden darf.
-(defun expand-LETF* (bindlist declare body)
+(defun expand-LETF* (bindlist declare body whole-form env)
   (if (atom bindlist)
     (if bindlist
       (error-of-type 'source-program-error
+        :form whole-form
+        :detail bindlist
         (TEXT "LETF* code contains a dotted list, ending with ~S")
         bindlist
       )
       (values `(LET* () ,@declare ,@body) t)
     )
     (let ((bind (car bindlist)) place form)
-      (if (atom bind) (setq place bind form nil)
+      (if (atom bind)
+        (setq place bind form nil)
         (if (and (consp (cdr bind)) (null (cddr bind)))
           (progn
             (setq place (car bind) form (cadr bind))
@@ -78,12 +82,16 @@
                 (return)
           ) ) )
           (error-of-type 'source-program-error
+            :form whole-form
+            :detail bind
             (TEXT "illegal syntax in LETF* binding: ~S")
             bind
       ) ) )
       (multiple-value-bind (rest-expanded flag)
-          (expand-LETF* (cdr bindlist) declare body)
-        (if (atom place)
+          (expand-LETF* (cdr bindlist) declare body whole-form env)
+        (if (and (atom place)
+                 (not (and (symbolp place) (nth-value 1 (macroexpand-1 place env))))
+            )
           (values
             (if flag
               `(LET* ,(cons (list place form) (cadr rest-expanded))
@@ -93,8 +101,12 @@
             )
             t
           )
-          (if (eq (car place) 'VALUES)
-            (if (every #'symbolp place)
+          (if (and (consp place) (eq (car place) 'VALUES))
+            (if (every #'(lambda (subplace)
+                           (and (symbolp subplace)
+                                (not (nth-value 1 (macroexpand-1 subplace env)))
+                         ) )
+                       place)
               (values
                 `(MULTIPLE-VALUE-BIND ,(cdr place) ,form ,@declare ,rest-expanded)
                 nil
@@ -114,41 +126,45 @@
                             (PROGN ,@(nreverse stores1) ,rest-expanded)
                             ,@(nreverse stores2)
                     ) ) ) )
-                  (multiple-value-bind (SM1 SM2 SM3 SM4 SM5)
-                      (get-setf-method (pop subplacesr))
+                  (multiple-value-bind (SM-temps SM-subforms SM-stores SM-setterform SM-getterform)
+                      (get-setf-method (pop subplacesr) env)
                     (setq bindlist
-                      (cons (list (first SM3) SM5)
-                            (nreconc (mapcar #'list SM1 SM2) bindlist)
+                      (cons (list (first SM-stores) SM-getterform)
+                            (nreconc (mapcar #'list SM-temps SM-subforms) bindlist)
                     ) )
-                    (let ((storetemp (gensym)))
+                    (let ((storetemp (gensym "LETF*-")))
                       (setq storetemps (cons storetemp storetemps))
-                      (setq stores1 (cons (subst storetemp (first SM3) SM4) stores1))
+                      ; We can use subst-in-form here, because storetemp is just a variable reference.
+                      (setq stores1 (cons (subst-in-form storetemp (first SM-stores) SM-setterform) stores1))
                     )
-                    (setq stores2 (cons SM4 stores2))
+                    (setq stores2 (cons SM-setterform stores2))
                 ) )
                 t
             ) )
-            (multiple-value-bind (SM1 SM2 SM3 SM4 SM5) (get-setf-method place)
-              (let ((formvar (gensym)))
+            (multiple-value-bind (SM-temps SM-subforms SM-stores SM-setterform SM-getterform)
+                (get-setf-method place env)
+              (let ((formvar (gensym "LETF*-VALUE-")))
                 (values
-                  `(LET* (,.(mapcar #'list SM1 SM2)
-                          (,(first SM3) ,SM5)
+                  `(LET* (,.(mapcar #'list SM-temps SM-subforms)
+                          (,(first SM-stores) ,SM-getterform)
                           (,formvar ,form))
                      ,@declare
                      (UNWIND-PROTECT
-                       (PROGN ,(subst formvar (first SM3) SM4) ,rest-expanded)
-                       ,SM4
+                       ; We can use subst-in-form here, because formvar is just a variable reference.
+                       (PROGN ,(subst-in-form formvar (first SM-stores) SM-setterform) ,rest-expanded)
+                       ,SM-setterform
                    ) )
                   t
             ) ) )
 ) ) ) ) ) )
 
-(defmacro LETF (bindlist &body body)
+(defmacro LETF (&whole whole-form
+                bindlist &body body &environment env)
   (multiple-value-bind (body-rest declarations) (SYSTEM::PARSE-BODY body)
     (let ((declare (if declarations `((DECLARE ,@declarations)) '()))
           (let-list nil))
       (multiple-value-bind (let*-list let/let*-list uwp-store1 uwp-store2)
-          (expand-LETF bindlist)
+          (expand-LETF bindlist whole-form env)
         ; mehrfach folgendes anwenden:
         ; endet let*-list mit (#:G form) und kommt in let/let*-list (var #:G)
         ; vor, so dürfen beide gestrichen werden, und dafür kommt (var form)
@@ -193,6 +209,7 @@
         (let ((body body-rest) ; eine Formenliste ohne Deklarationen
               (1form nil)) ; zeigt an, ob body aus einer einzigen Form besteht
           (when uwp-store1
+            (unless body (setq body '(nil)))
             (setq body `((UNWIND-PROTECT (PROGN ,@uwp-store1 ,@body) ,@uwp-store2))
                   1form t
           ) )
@@ -213,21 +230,25 @@
 ) ) ) ) ) )
 
 ; expandiert ein LETF, liefert:
-; eine Bindungsliste für LETF*,
-; eine Bindungsliste für LET/LET* (Reihenfolge der Bindung darin beliebig),
-; eine Liste von Bindungsanweisungen, eine Liste von Entbindungsanweisungen
+; 1. eine Bindungsliste für LETF*,
+; 2. eine Bindungsliste für LET/LET* (Reihenfolge der Bindung darin beliebig),
+; 3. eine Liste von Bindungsanweisungen,
+; 4. eine Liste von Entbindungsanweisungen
 ; (beide gleich lang).
-(defun expand-LETF (bindlist)
+(defun expand-LETF (bindlist whole-form env)
   (if (atom bindlist)
     (if bindlist
       (error-of-type 'source-program-error
+        :form whole-form
+        :detail bindlist
         (TEXT "LETF code contains a dotted list, ending with ~S")
         bindlist
       )
       (values '() '() '() '())
     )
     (let ((bind (car bindlist)) place form)
-      (if (atom bind) (setq place bind form nil)
+      (if (atom bind)
+        (setq place bind form nil)
         (if (and (consp (cdr bind)) (null (cddr bind)))
           (progn
             (setq place (car bind) form (cadr bind))
@@ -240,19 +261,26 @@
                 (return)
           ) ) )
           (error-of-type 'source-program-error
+            :form whole-form
+            :detail bind
             (TEXT "illegal syntax in LETF binding: ~S")
             bind
       ) ) )
-      (multiple-value-bind (L1 L2 L3 L4) (expand-LETF (cdr bindlist))
-        (if (atom place)
+      (multiple-value-bind (L1 L2 L3 L4) (expand-LETF (cdr bindlist) whole-form env)
+        (if (and (atom place)
+                 (not (and (symbolp place) (nth-value 1 (macroexpand-1 place env))))
+            )
           (let ((g (gensym)))
             (values (cons (list g form) L1) (cons (list place g) L2) L3 L4)
           )
-          (if (eq (car place) 'VALUES)
-            (if (every #'symbolp place)
+          (if (and (consp place) (eq (car place) 'VALUES))
+            (if (every #'(lambda (subplace)
+                           (and (symbolp subplace)
+                                (not (nth-value 1 (macroexpand-1 subplace env)))
+                         ) )
+                       place)
               (let ((gs (mapcar #'(lambda (subplace)
-                                    (declare (ignore subplace))
-                                    (gensym)
+                                    (gensym (symbol-name subplace))
                                   )
                                 (cdr place)
                    ))   )
@@ -270,31 +298,37 @@
                   ((atom subplacesr)
                    (values
                      (nreconc bindlist
-                              (cons (list (cons 'VALUES storetemps) form) L1)
+                              (cons (list (cons 'VALUES (nreverse storetemps)) form) L1)
                      )
                      L2
                      (nreconc stores1 L3)
                      (nreconc stores2 L4)
                   ))
-                (multiple-value-bind (SM1 SM2 SM3 SM4 SM5)
-                    (get-setf-method (pop subplacesr))
+                (multiple-value-bind (SM-temps SM-subforms SM-stores SM-setterform SM-getterform)
+                    (get-setf-method (pop subplacesr) env)
                   (setq bindlist
-                    (cons (list (first SM3) SM5)
-                          (nreconc (mapcar #'list SM1 SM2) bindlist)
+                    (cons (list (first SM-stores) SM-getterform)
+                          (nreconc (mapcar #'list SM-temps SM-subforms) bindlist)
                   ) )
-                  (let ((storetemp (gensym)))
+                  (let ((storetemp (gensym "LETF-")))
                     (setq storetemps (cons storetemp storetemps))
-                    (setq stores1 (cons (subst storetemp (first SM3) SM4) stores1))
+                    ; We can use subst-in-form here, because storetemp is just a variable reference.
+                    (setq stores1 (cons (subst-in-form storetemp (first SM-stores) SM-setterform) stores1))
                   )
-                  (setq stores2 (cons SM4 stores2))
+                  (setq stores2 (cons SM-setterform stores2))
             ) ) )
-            (multiple-value-bind (SM1 SM2 SM3 SM4 SM5) (get-setf-method place)
-              (let ((g (gensym)))
+            (multiple-value-bind (SM-temps SM-subforms SM-stores SM-setterform SM-getterform)
+                (get-setf-method place env)
+              (let ((g (gensym "LETF-VALUE-")))
                 (values
-                  `(,.(mapcar #'list SM1 SM2) (,(first SM3) ,SM5) (,g ,form))
+                  `(,.(mapcar #'list SM-temps SM-subforms)
+                    (,(first SM-stores) ,SM-getterform)
+                    (,g ,form)
+                    . ,L1)
                   L2
-                  (cons (subst g (first SM3) SM4) L3)
-                  (cons SM4 L4)
+                  ; We can use subst-in-form here, because g is just a variable reference.
+                  (cons (subst-in-form g (first SM-stores) SM-setterform) L3)
+                  (cons SM-setterform L4)
             ) ) )
 ) ) ) ) ) )
 
@@ -334,3 +368,16 @@ on other lisps (which is 1.5-2 times as fast as push/nreverse there)."
                          collectors ret tail tmp)
         ,@forms
         (values ,@ret)))))
+
+;;; ---------------------------------------------------------------------------
+(defun compiled-file-p (file-name)
+  "Return non-NIL is FILE-NAME names a CLISP-compiled file
+with compatible bytecodes."
+  (with-open-file (in file-name :direction :input :if-does-not-exist nil)
+    (handler-bind ((error (lambda (c) (declare (ignore c))
+                                  (return-from compiled-file-p nil))))
+      (and in (char= #\( (peek-char nil in nil #\a))
+           (let ((form (read in nil nil)))
+             (and (consp form)
+                  (eq (car form) 'SYSTEM::VERSION)
+                  (null (eval form))))))))
